package com.xhkjedu.smarking.controller.paper;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.annotation.Action;
import com.xhkjedu.smarking.model.exam.MsSubject;
import com.xhkjedu.smarking.model.paper.MsPaperBack;
import com.xhkjedu.smarking.service.paper.MsPaperService;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @Description 阅卷试卷表相关操作
 * @Author auto
 * @Date 2024-10-15
 */
@RestController
@RequestMapping("/mp")
public class MsPaperController {
    @Resource
    private MsPaperService msPaperService;

    /*
     * @Description 教师出卷列表
     * @Date 2024/10/18 10:52:58
     * @Author WN
     * @Param [subject]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/list_teacher")
    public ResultVo examTeacherPaperList(@RequestBody MsSubject subject){
        Integer page = subject.getPage();
        Integer pageSize = subject.getPageSize();
        N_Utils.validation(new Object[]{subject.getTeacherid(),"教师id",1,page,"显示页码",1,pageSize,"显示条数",1});
        PageHelper.startPage(page, pageSize);
        List<Map<String,Object>> rtnlst = msPaperService.listExamPapersByTeacherid(subject.getTeacherid(), subject.getPstate());
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(rtnlst));
        return new ResultVo(0,"成功获取教师出卷信息",pageResult);
    }

    /*
     * @Description 试卷--退回
     * @Date 2024/10/18 11:02:17
     * @Author WN
     * @Param [paperBack]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/back_save")
    @Action("考试测评-退回试卷")
    public ResultVo saveExamPaperBack(@RequestBody MsPaperBack paperBack){
        N_Utils.validation(new Object[]{paperBack.getMpid(),"试卷id",1});
        msPaperService.savePaperBack(paperBack);
        return new ResultVo(0,"成功退回试卷",null);
    }

    /*
     * @Description 试卷--退回原因列表
     * @Date 2024/10/18 11:02:22
     * @Author WN
     * @Param [paperBack]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/back_list")
    public ResultVo listPaperBacks(@RequestBody MsPaperBack paperBack){
        N_Utils.validation(new Object[]{paperBack.getMpid(),"试卷id",1});
        List<Map<String,Object>> list = msPaperService.listPaperBacksByEpid(paperBack.getMpid());
        return new ResultVo(0,"成功获取试卷退回原因",list);
    }
}
