package com.xhkjedu.smarking.service.paper;

import com.xhkjedu.exception.MissingParametersException;
import com.xhkjedu.smarking.mapper.exam.MsSubjectMapper;
import com.xhkjedu.smarking.mapper.paper.MsPaperBackMapper;
import com.xhkjedu.smarking.mapper.paper.MsPaperMapper;
import com.xhkjedu.smarking.model.paper.MsPaperBack;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @Description 阅卷试卷表 服务实现类
 * @Author auto
 * @Date 2024-10-15
 */
@Service
public class MsPaperService {
    @Resource
    private MsPaperMapper msPaperMapper;
    @Resource
    private MsPaperBackMapper msPaperBackMapper;
    @Resource
    private MsSubjectMapper msSubjectMapper;

    /*
     * @Description 列表--教师出卷
     * @Date 2024/10/18 10:52:31
     * @Author WN
     * @Param [teacherid, esstate]
     * @Return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     **/
    public List<Map<String,Object>> listExamPapersByTeacherid(Integer teacherid, Integer pstate){
        int timestamp = N_Utils.getSecondTimestamp();
        String strtime = N_Utils.getStrtimeTimestamp(timestamp, "yyyy-MM-dd HH:mm:ss");
        return msPaperMapper.listExamPapersByTeacherid(teacherid,pstate,strtime);
    }

    /*
     * @Description 试卷--退回
     * @Date 2024/10/18 11:00:57
     * @Author WN
     * @Param [paperBack]
     * @Return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public void savePaperBack(MsPaperBack paperBack) {
        Integer examstate = msPaperMapper.getExamStateByMpid(paperBack.getMpid());
        if (examstate == 1 ) throw new MissingParametersException("考试已开始禁止操作");
        if (examstate >= 2 ) throw new MissingParametersException("考试已结束禁止操作");
        int timestamp = N_Utils.getSecondTimestamp();
        paperBack.setCreatetime(timestamp);
        msPaperBackMapper.insertUseGeneratedKeys(paperBack);
        //更改试卷科目状态
        msSubjectMapper.updatePstateByMpid(3, paperBack.getMpid());
    }

    /*
     * @Description 试卷--退回原因列表
     * @Date 2024/10/18 11:01:11
     * @Author WN
     * @Param [mpid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map<String,Object>> listPaperBacksByEpid(Integer mpid){
        return msPaperBackMapper.listPaperBacksByMpid(mpid);
    }

    //获取考试ID
    public Integer getExamidByMpid(Integer mpid){
        return msPaperMapper.getExamidByMpid(mpid);
    }

}
