package com.xhkjedu.smarking.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.xhkjedu.smarking.config.ConfigKey;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;

import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author ywx
 * @className ClassUtil
 * @description
 * @date 2022/2/21 17:20
 **/
@Slf4j
public class MarkingUtil {
    public static Map<String, String> uriMap = new ConcurrentHashMap<>();

    /**
     * 删除Html标签
     *
     */
    public static String htmlRemoveTag(String inputString) {
        if (inputString == null)
            return null;
        String htmlStr = inputString; // 含html标签的字符串
        String textStr = "";
        java.util.regex.Pattern p_script;
        java.util.regex.Matcher m_script;
        java.util.regex.Pattern p_style;
        java.util.regex.Matcher m_style;
        java.util.regex.Pattern p_html;
        java.util.regex.Matcher m_html;
        try {
            //定义script的正则表达式{或<script[^>]*?>[\\s\\S]*?<\\/script>
            String regEx_script = "<[\\s]*?script[^>]*?>[\\s\\S]*?<[\\s]*?\\/[\\s]*?script[\\s]*?>";
            //定义style的正则表达式{或<style[^>]*?>[\\s\\S]*?<\\/style>
            String regEx_style = "<[\\s]*?style[^>]*?>[\\s\\S]*?<[\\s]*?\\/[\\s]*?style[\\s]*?>";
            String regEx_html = "<[^>]+>"; // 定义HTML标签的正则表达式
            p_script = Pattern.compile(regEx_script, Pattern.CASE_INSENSITIVE);
            m_script = p_script.matcher(htmlStr);
            htmlStr = m_script.replaceAll(""); // 过滤script标签
            p_style = Pattern.compile(regEx_style, Pattern.CASE_INSENSITIVE);
            m_style = p_style.matcher(htmlStr);
            htmlStr = m_style.replaceAll(""); // 过滤style标签
            p_html = Pattern.compile(regEx_html, Pattern.CASE_INSENSITIVE);
            m_html = p_html.matcher(htmlStr);
            htmlStr = m_html.replaceAll(""); // 过滤html标签
            textStr = htmlStr;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return textStr;// 返回文本字符串
    }

    /*
     * @Description 判断两个字符串的关系
     * @Date 2024/10/29 10:23:58
     * @Author WN
     * @Param [str1, str2]
     * @Return int
     **/
    public static int isEqualStringCheck(String str1,String str2){
        char[] charArray1 = str1.toCharArray();
        char[] charArray2 = str2.toCharArray();
        Arrays.sort(charArray1);
        Arrays.sort(charArray2);
        int eqResult = 0;//0代表不相等 1表示两个字符串相等 2代表字符串1包含字符串2
        if(Arrays.equals(charArray1, charArray2)){
            eqResult = 1;
        }

        if(Collections.singletonList(charArray1).contains(charArray2)){
            eqResult = 2;
        }

        return eqResult;
    }

    /*
     * @Description 客观题自动批阅
     * @Date 2024/11/5 11:18:30
     * @Author WN
     * @Param [stuanswer, q]
     * @Return java.lang.Double
     **/
    public static Double checkObjectiveStuAnswer(String stuanswer, int ctype,String qanswer,Double loseoption,Double qscore){
        String qanswerCort = JSON.parseArray(qanswer, String.class).get(0);//试题正确答案
        Double stuscore = 0D;
        if(N_Utils.isNotEmpty(stuanswer)){
            if(ctype == 1 || ctype == 4 || ctype == 5 || ctype == 6){
                if(qanswerCort.equalsIgnoreCase(stuanswer)){
                    stuscore = qscore;
                }
            }else if(ctype == 2){
                //多选题进行判断，
                int isEqualStr = isEqualStringCheck(qanswerCort, stuanswer);
                if(isEqualStr == 1){
                    stuscore = qscore;
                }else if(isEqualStr == 2){
                    stuscore = loseoption;//少选得分
                }
            }
        }
        return stuscore;
    }


    /**
     * 提供精确的加法运算。
     *
     * @param v1 被加数
     * @param v2 加数
     * @return 两个参数的和
     */
    public static double add(Object v1, Object v2) {
        BigDecimal b1 = new BigDecimal(v1.toString());
        BigDecimal b2 = new BigDecimal(v2.toString());
        return b1.add(b2).doubleValue();
    }

    //两数相减
    public static long subtract(long a, long b) {
        BigInteger biginta = new BigInteger(a + "");
        BigInteger bigintb = new BigInteger(b + "");
        return biginta.subtract(bigintb).longValue();

    }

    //两数相减
    public static Integer subtractInt(Integer a, Integer b) {
        BigInteger biginta = new BigInteger(a + "");
        BigInteger bigintb = new BigInteger(b + "");
        return biginta.subtract(bigintb).intValue();

    }

    /**
     * 提供精确的乘法运算。
     *
     * @param v1 被乘数
     * @param v2 乘数
     * @return 两个参数的积
     */
    public static double mul(double v1, double v2) {
        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));
        return b1.multiply(b2).doubleValue();
    }

    /**
     * 提供（相对）精确的除法运算，当发生除不尽的情况时，精确到 小数点以后2位，以后的数字舍弃。
     * @param v1 被除数
     * @param v2 除数
     * @return 两个参数的商
     */
    public static double div(Object v1, Object v2) {
        BigDecimal b1 = new BigDecimal(v1.toString());
        BigDecimal b2 = new BigDecimal(v2.toString());
        return b1.divide(b2, 2, BigDecimal.ROUND_DOWN).doubleValue();
    }

    /**
     * @Description 计算平均分
     * @Date 2024/11/19 11:30
     * @Author YWX
     * @Param [v1, v2]
     * @Return double
     **/
    public static double avg(Object v1, Object v2) {
        return div(add(v1, v2), 2);
    }

    /**
     * 向上取值
     * @param score 原始得分
     * @param interval 取值间隔
     * @return 向上取值后的结果
     */
    public static double roundUp(double score, double interval) {
        return Math.ceil(score / interval) * interval;
    }

    /**
     * 向下取值
     * @param score 原始得分
     * @param interval 取值间隔
     * @return 向下取值后的结果
     */
    public static double roundDown(double score, double interval) {
        return Math.floor(score / interval) * interval;
    }

    /**
     * 计算总分并处理超过满分的情况
     * @param scores 得分数组
     * @param interval 取值间隔
     * @param maxScore 满分
     * @return 最终总分
     */
    public static double calculateTotalScore(double[] scores, double interval, double maxScore) {
        double totalScore = 0.0;

        for (double score : scores) {
            totalScore += roundUp(score, interval);
        }

        return Math.min(totalScore, maxScore);
    }

    /**
     * 计算两个分数之间的分差
     * @param score1 第一个分数
     * @param score2 第二个分数
     * @return 分差
     */
    public static double scoreDiff(Object score1, Object score2) {
        return Math.abs(sub(score1, score2));
    }


    /**
     * 提供精确的减法运算。
     *
     * @param v1 被减数
     * @param v2 减数
     * @return 两个参数的差
     */
    public static double sub(Object v1, Object v2) {
        BigDecimal b1 = new BigDecimal(v1.toString());
        BigDecimal b2 = new BigDecimal(v2.toString());
        return b1.subtract(b2).doubleValue();
    }

    //字符串转为时间戳(秒)
    public static long strToTimestamp(String str) {
        if (str == null || str.isEmpty()) return 0;
        DateTimeFormatter ftf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime parse = LocalDateTime.parse(str, ftf);
        return LocalDateTime.from(parse).atZone(ZoneId.systemDefault()).toEpochSecond();
    }

    /**
     * @Description 字符串转时间戳
     * @Param [time]
     * @Return int
     * @Author wn
     * @Date 2022/10/27 11:36
     **/
    public static long getTimestrToTimetamp(String time){
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = new Date();
        try {
            date = sdf.parse(time);
        }catch (Exception e){
            log.error("字符串转时间戳出错：" + e.getMessage());
        }
        return date.getTime()/1000;
    }

    /*
     * @Description base64图片上传
     * @Date 2024/10/30 10:57:20
     * @Author WN
     * @Param [stupic, merged, saveFloder]
     * @Return java.lang.String
     **/
    public static String uploadBase64Img(String stupic,Integer merged,String saveFloder) {
        String apiMethod;
        if(merged == 1){
            //说明上传的图片为单张保存
            apiMethod = "bs/base";
        }else{
            //上传的多张图片需要合并为一张
            apiMethod = "bs/mbase_merge";
        }
        String stubase64;//防止内容被更改
        try {
            stubase64 = URLEncoder.encode(stupic, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            log.error("base64格式化内容出错" + e.getMessage());
            throw new RuntimeException(e);
        }
        //处理字段那种图片路径,base64转图片
        String params = "base64path=" + stubase64 + "&savefolder=" + saveFloder;
        String fileInfoStr = N_Utils.sendPost(ConfigKey.fileapi + apiMethod, params);
        if(N_Utils.isEmpty(fileInfoStr)){
            return "";
        }
        ResultVo fileResult = JSON.parseObject(fileInfoStr, ResultVo.class);
        if (fileResult.getCode() == 0 && N_Utils.isNotEmpty(fileResult.getObj())) {
            return fileResult.getObj().toString();
        }else{
            return "";
        }
    }

    /*
     * @Description 批量上传base64图片
     * @Date 2024/10/30 15:09:05
     * @Author WN
     * @Param [stupic, saveFloder]
     * @Return java.util.List<java.lang.String>
     **/
    public static List<String> uploadBatchBase64Img(List<String> imgList,String saveFloder){
        //处理字段那种图片路径,base64转图片
        String params = "filepaths=" + imgList + "&savefolder=" + saveFloder;
        String fileInfoStr = N_Utils.sendPost(ConfigKey.fileapi + "bs/mbase", params);
        ResultVo fileResult = JSON.parseObject(fileInfoStr, ResultVo.class);
        if (fileResult.getCode() == 0 && N_Utils.isNotEmpty(fileResult.getObj())) {
            return JSONArray.parseArray(fileResult.getObj().toString(), String.class);
        }else{
            return new ArrayList<>();
        }
    }

    /*
     * @Description 识别客观题题块单题结果
     * @Date 2024/11/7 14:58:44
     * @Author WN
     * @Param [input]
     * @Return java.util.List<java.lang.String>
     **/
    public static List<String> extractCharacters(String input) {
        List<String> result = new ArrayList<>();
        Pattern pattern = Pattern.compile("\\((.*?)\\)|([^()]+)");
        Matcher matcher = pattern.matcher(input);

        while (matcher.find()) {
            if (matcher.group(1)!= null) {
                result.add(matcher.group(1));
            } else if (matcher.group(2)!= null) {
                String str = matcher.group(2);
                for(char c : str.toCharArray()){
                    result.add(c+"");
                }
            }
        }

        return result;
    }

    public static void main(String[] args) {
        String str = "ADAC-D-(C)(AB)(CD)(ACB)BB";

        List<String> result = extractCharacters(str);

        for (String s : result) {
            System.out.println(s);
        }
    }


}
